#ifndef _Variable_number_of_arguments_H
#define _Variable_number_of_arguments_H

#include <ctime>

#include <set>

class Human_being {
private:
    struct tm _birth_date;
public:
    Human_being(int = 1, int = 0, int = 1970 - 1900);
    int age() const;
    // Old style:
    void births(Human_being*...);
    void births(int, Human_being*...);
public: // Must be 'public:' when called with one argument (direct call!):

    template <typename Child> void variadic_births(Child child) { // Base case...
        // Template member function in essence allows 'this':
        this->_children.insert(child); // This ends recursion...
    }

    template<typename Child, typename... Children> void variadic_births(Child child, Children... children) { // "Parameter pack"
        this->variadic_births(child);
        this->variadic_births(children...); // Unpack
    };

    const Human_being cloning() const;
    size_t count() const;
    void insert(Human_being*); // Useless: for variadic non-member function illustration only!
private:
    std::set<Human_being*> _children;
};

class Man : public Human_being {
public:
    const Man cloning() const;
};

class Woman : public Human_being {
public:
    const Woman cloning() const;
};

template <typename... Member> class Family {
};

namespace variadic_template {

    template <typename T> // Simplification: 'parent' and 'child' come from the same type
    void births(T& parent, T* child) {
        // Constraints on 'T' is powered from C++20 (https://en.cppreference.com/w/cpp/language/constraints)
        static_assert(std::is_base_of<Human_being, T>::value, "Type parameter of this class must derive from 'Human_being'");
        parent.insert(child); // Call of member function to access '_children'...
    }

    template<typename T, typename... Children>
    void births(T& parent, T* child, Children... children) {
        static_assert(sizeof... (Children) > 0, "Never inferred with '0' arguments, end of unpacking...");
        // Constraints on 'T' is powered from C++20 (https://en.cppreference.com/w/cpp/language/constraints)
        static_assert(std::is_base_of<Human_being, T>::value, "Type parameter of this class must derive from 'Human_being'");
        births(parent, child);
        births(parent, children...);
    }
}

#endif /* _Variable_number_of_arguments_H */

